"use client";
import React, { useEffect, useState } from "react";

import { notFound, useParams } from "next/navigation";

import { Component, ContainerComponent, PageData } from "@/types/componentTypes";
import { componentRegistry, containerComponents, widthClasses } from "@/utils/componentRegistry";

/**
 * Function to check if a component is a container component.
 */
function isContainerComponent(component: Component): component is ContainerComponent {
  return 'children' in component;
}

interface ComponentWrapperProps {
  component: Component;
  parent?: Component;
  children?: React.ReactNode;
}

function ComponentWrapper({ component, parent, children }: ComponentWrapperProps) {
  const ComponentToRender = componentRegistry[component.type];
  const parentIsRow = parent?.type === "row";
  const siblingCount = parentIsRow ? parent.children?.length ?? 1 : 1;
  const dynamicWidth = parentIsRow
    ? widthClasses[siblingCount as keyof typeof widthClasses] || "w-full"
    : "w-full";

  if (component.type === "column") {
    return (
      <div key={component.id} className={`w-full px-4 md:${dynamicWidth}`}>
        <ComponentToRender imageHero={""} title={""} width={""} text={""} src={""} alt={""} url={""} embedId={""} image={""} {...component.props}>
          {children}
        </ComponentToRender>
      </div>
    );
  }

  return (
    <ComponentToRender imageHero={""} title={""} width={""} text={""} src={""} alt={""} url={""} embedId={""} image={""} {...component.props}>
      {children}
    </ComponentToRender>
  );
}

function ComponentRenderer({ components, parent }: { components: Component[]; parent?: Component }) {
  return (
    <>
      {components.map((component) => (
        <ComponentWrapper key={component.id} component={component} parent={parent}>
          {containerComponents.includes(component.type) && isContainerComponent(component) && component.children && (
            <ComponentRenderer components={component.children} parent={component} />
          )}
        </ComponentWrapper>
      ))}
    </>
  );
}

export default function PageView() {
  const { slug } = useParams() as { slug: string };
  const [pageData, setPageData] = useState<PageData | null>(null);
  const [loading, setLoading] = useState(true);
  const [error, setError] = useState<string | null>(null);

  useEffect(() => {
    console.log("Fetching page data for slug:", slug);
    if (!slug) {
      return notFound();
    };


    fetch(`/api/pages/${slug}`)
    .then((res) => res.json())
    .then((data) => {
      if (data.success) {
        setPageData(data.data);
        setLoading(false);
      } else {
        setError(data.error || "Page not found");
        setLoading(false);
      }
    })
    .catch(() => {
      setError("Failed to load page");
      setLoading(false);
    });
  
  }, [slug]);

  if (loading) return <p className="text-gray-500">Loading...</p>;
  if (error) {
    if (error === "Not Found") {
      return notFound();
    } else {
      if (loading) return <p className="text-red-500">{error}</p>;
    }
  }

  return (
    <main className="min-h-screen">
      {pageData?.components && pageData.components.length > 0 ? (
        <ComponentRenderer components={pageData.components} />
      ) : (
        <div className="container mx-auto py-8">
          <div className="bg-yellow-100 border border-yellow-400 text-yellow-700 px-4 py-3 rounded">
            <div className="flex items-center">
              <span className="font-bold mr-2">Empty Page</span>
            </div>
            <p>This page has no content components.</p>
          </div>
        </div>
      )}
    </main>
  );
}