import type { NextAuthOptions } from "next-auth";
import GitHubProvider from "next-auth/providers/github";
import CredentialsProvider from "next-auth/providers/credentials";
import bcrypt from "bcryptjs";
import connectToDatabase from "@/utils/mongoose";
import type { IUser } from "@/models/User";
import User from "@/models/User";

import type { JWT } from "next-auth/jwt";
import type { Session } from "next-auth";

declare module "next-auth" {
    interface Session {
        user: {
            id: string;
            name?: string | null;
            email?: string | null;
            image?: string | null;
            role?: string;
        };
    }
}

// NextAuth Configuration
export const options: NextAuthOptions = {

    // Autentication providers
    providers: [
        // Github provider
        GitHubProvider({
            clientId: process.env.GITHUB_ID as string,
            clientSecret: process.env.GITHUB_SECRET as string,
            async profile(profile) {
                await connectToDatabase();
                let user = await User.findOne({ email: profile.email });

                // If the user don't exist, create one
                if (!user) {
                    const isFirstUser = (await User.countDocuments()) === 0;
                    user = await User.create({
                        name: profile.name || profile.login,
                        email: profile.email,
                        provider: "github",
                        role: isFirstUser ? "admin" : "visitor", // First user is admin
                    });
                }

                // Return the user information
                return {
                    id: user.id,
                    name: user.name,
                    email: user.email,
                    role: user.role,
                };
            }
        }),
        // Credentials provider
        CredentialsProvider({
            name: "Credentials",
            credentials: {
                identifier: {
                    label: "Email or Username",
                    type: "text",
                    placeholder: "jsmith or jsmith@example.com"
                },
                password: { label: "Password", type: "password" },
            },
            async authorize(credentials) {
                await connectToDatabase();

                // Search user by email or username
                const user = await User.findOne({
                    $or: [
                        { email: credentials?.identifier },
                        { name: credentials?.identifier }
                    ]
                }).select("+password");

                // If the user don't exist, throw an error
                if (!user || !user.password) {
                    console.log(user.password)
                    throw new Error("Invalid email or password");
                }

                // Comparar contraseñas encriptadas
                const isPasswordValid = credentials?.password
                    ? await bcrypt.compare(credentials.password, user.password)
                    : false;
                if (!isPasswordValid) {
                    throw new Error("Invalid email or password");
                }

                // Return user information
                return user;
            }
        })
    ],
    // Custom pages
    pages: {
        signIn: "/auth/signin", // Sign in page
        signOut: "/auth/signout", // Sign out page
        error: "/auth/error", // Error page
        verifyRequest: "/auth/verify-request", // Verify request page
    },
    // Session configuration
    session: {
        strategy: "jwt",
        maxAge: 86400 // one day session
    },
    // Custom callbacks
    callbacks: {
        async jwt({ token, user }) {
            if (user) {
                token.id = user.id;
                token.role = (user as IUser).role;
            }
            return token;
        },
        async session({ session, token }: { session: Session; token: JWT }) {
            if (session.user) {
                session.user.id = token.id as string;
                session.user.role = token.role as string;
            }
            return session;
        }
    },
    // Secret environment variable
    secret: process.env.NEXTAUTH_SECRET,
};