import { NextRequest, NextResponse } from "next/server";
import { writeFile, readdir, stat, unlink } from "fs/promises";
import path from "path";
import { authenticateUser } from "@/utils/auth";
import { logError } from "@/utils/logger";

// Allowed file types and their corresponding extensions
const ALLOWED_TYPES = {
  'image/jpeg': 'jpg',
  'image/png': 'png',
  'image/gif': 'gif',
  'application/pdf': 'pdf'
};

const MAX_FILE_SIZE = 10 * 1024 * 1024; // 10MB

// Helper to get Mime type
function getMimeType(extension: string): string {
  const mimeTypes: { [key: string]: string } = {
    'jpg': 'image/jpeg',
    'jpeg': 'image/jpeg',
    'webp': 'image/webp',
    'png': 'image/png',
    'gif': 'image/gif',
    'pdf': 'application/pdf'
  };
  return mimeTypes[extension] || 'application/octet-stream';
}

// Helper to sanitize file name
function sanitizeFileName(fileName: string): string {
  // Replace spaces with underscores
  return fileName
    .replace(/\s+/g, '_')
    // Delete special characters
    .replace(/[^a-zA-Z0-9\-_\.]/g, '')
    // Convert to lowercase
    .toLowerCase();
}

// Function to split file name and extension
function splitFileName(fileName: string): { baseName: string; extension: string } {
  const lastDotIndex = fileName.lastIndexOf('.');
  if (lastDotIndex === -1) {
    return { baseName: fileName, extension: '' };
  }
  return {
    baseName: fileName.substring(0, lastDotIndex),
    extension: fileName.substring(lastDotIndex + 1)
  };
}

// Function to generate a unique file name
async function generateUniqueFileName(
  originalName: string,
  uploadsDir: string
): Promise<string> {
  // Sanitize the file name
  const sanitizedName = sanitizeFileName(originalName);
  const { baseName, extension } = splitFileName(sanitizedName);
  
  // Get the list of existing files
  const existingFiles = await readdir(uploadsDir);
  
  // If the file name is unique, return it
  if (!existingFiles.includes(sanitizedName)) {
    return sanitizedName;
  }
  
  // Find a unique file name by adding a counter
  let counter = 2;
  let newFileName: string;
  
  do {
    newFileName = `${baseName}${counter}.${extension}`;
    counter++;
  } while (existingFiles.includes(newFileName));
  
  return newFileName;
}

// GET: List files
export async function GET() {
  try {
    const uploadsDir = path.join(process.cwd(), 'public', 'uploads');
    const files = await readdir(uploadsDir);

    // Get detailed information for each file
    const filesInfo = await Promise.all(
      files.map(async (fileName) => {
        const filePath = path.join(uploadsDir, fileName);
        const fileStats = await stat(filePath);
        const extension = path.extname(fileName).slice(1);
        
        return {
          name: fileName,
          url: `/uploads/${fileName}`,
          size: fileStats.size,
          type: getMimeType(extension),
          createdAt: fileStats.birthtime,
          updatedAt: fileStats.mtime
        };
      })
    );

    // Order by creation date
    filesInfo.sort((a, b) => b.createdAt.getTime() - a.createdAt.getTime());

    return NextResponse.json({
      success: true,
      data: filesInfo,
      message: 'Files retrieved successfully'
    });

  } catch (error) {
    logError(error as Error, "GET /api/upload");
    return NextResponse.json(
      { success: false, error: 'Failed to retrieve files' },
      { status: 500 }
    );
  }
}

// POST: Upload file
export async function POST(req: NextRequest) {
  try {
    const auth = await authenticateUser(req);
    if (!auth.authorized) return auth.response;

    const formData = await req.formData();
    const file = formData.get('file') as File;

    if (!file) {
      return NextResponse.json(
        { success: false, error: 'No file provided' },
        { status: 400 }
      );
    }

    // Validar tipo de archivo
    if (!Object.keys(ALLOWED_TYPES).includes(file.type)) {
      return NextResponse.json(
        { success: false, error: 'File type not allowed' },
        { status: 400 }
      );
    }

    // Validar tamaño
    if (file.size > MAX_FILE_SIZE) {
      return NextResponse.json(
        { success: false, error: 'File size exceeds 5MB limit' },
        { status: 400 }
      );
    }

    const uploadsDir = path.join(process.cwd(), 'public', 'uploads');
    
    // Generar nombre único preservando el nombre original
    const fileName = await generateUniqueFileName(file.name, uploadsDir);
    const filePath = path.join(uploadsDir, fileName);

    // Convertir el archivo a Buffer y guardarlo
    const bytes = await file.arrayBuffer();
    const buffer = Buffer.from(bytes);
    await writeFile(filePath, buffer);

    // Obtener estadísticas del archivo
    const fileStats = await stat(filePath);

    return NextResponse.json({
      success: true,
      data: {
        name: fileName,
        url: `/uploads/${fileName}`,
        type: file.type,
        size: fileStats.size,
        createdAt: fileStats.birthtime,
        updatedAt: fileStats.mtime
      },
      message: 'File uploaded successfully'
    });

  } catch (error) {
    logError(error as Error, "POST /api/upload");
    return NextResponse.json(
      { success: false, error: 'Failed to upload file' },
      { status: 500 }
    );
  }
}

// DELETE: Eliminar archivo
export async function DELETE(req: NextRequest) {
  try {
    const auth = await authenticateUser(req);
    if (!auth.authorized) return auth.response;

    // Obtener el nombre del archivo de la URL
    const searchParams = req.nextUrl.searchParams;
    const fileName = searchParams.get('fileName');

    if (!fileName) {
      return NextResponse.json(
        { success: false, error: 'File name is required' },
        { status: 400 }
      );
    }

    const uploadsDir = path.join(process.cwd(), 'public', 'uploads');
    const filePath = path.join(uploadsDir, fileName);

    // Verificar que el archivo existe
    try {
      await stat(filePath);
    } catch {
      return NextResponse.json(
        { success: false, error: 'File not found' },
        { status: 404 }
      );
    }

    // Eliminar el archivo
    await unlink(filePath);

    return NextResponse.json({
      success: true,
      message: 'File deleted successfully'
    });

  } catch (error) {
    logError(error as Error, "DELETE /api/media");
    return NextResponse.json(
      { success: false, error: 'Failed to delete file' },
      { status: 500 }
    );
  }
}