import { NextResponse, NextRequest } from 'next/server';
import connectToDatabase from '@/utils/mongoose';
import Page from '@/models/Page';
import { z } from 'zod';

import { ApiResponse, IPage } from '@/types';
import { authenticateUser } from '@/utils/auth';
import { logError } from '@/utils/logger';

// Validation schemas
const componentSchema = z.any();

const pageSchema = z.object({
  title: z.string()
    .min(1, 'Title is required')
    .transform((val) => val.replace(/\s+/g, '_')),
  createdAt: z.string(),
  components: z.array(componentSchema),
  updatedAt: z.string().optional(),
  publishedAt: z.string().optional(),
  createdBy: z.string(),
  status: z.enum(['draft', 'published']).optional().default('draft'),
});

const pageUpdateSchema = pageSchema.partial().extend({
  id: z.string()
});

// Helper function for standardized responses
function createResponse<T>(
  data?: T,
  message?: string,
  error?: string,
  status: number = 200
): NextResponse {
  const response: ApiResponse<T> = {
    success: !error,
    ...(data && { data }),
    ...(message && { message }),
    ...(error && { error })
  };
  return NextResponse.json(response, { status });
}

// Get all pages
export async function GET(req: NextRequest) {
  try {
    const auth = await authenticateUser(req);
    if (!auth.authorized) return auth.response; // ✅ Use authentication

    await connectToDatabase();
    const { searchParams } = new URL(req.url);

    // Parse query parameters
    const status = searchParams.get('status');
    const limit = parseInt(searchParams.get('limit') || '10');
    const page = parseInt(searchParams.get('page') || '1');

    // Build query
    const query: Record<string, unknown> = {};

    if (status) query.status = status;

    // Execute query with pagination
    const skip = (page - 1) * limit;
    const [pages, total] = await Promise.all([
      Page.find(query)
        .sort({ createdAt: -1 })
        .skip(skip)
        .limit(limit)
        .lean(),
      Page.countDocuments(query)
    ]);

    return createResponse({
      pages,
      pagination: {
        total,
        page,
        limit,
        pages: Math.ceil(total / limit)
      }
    }, 'Pages retrieved successfully');
  } catch (error) {
    logError(error as Error, "GET all pages /api/pages/");
    return createResponse(undefined, undefined, 'Failed to fetch pages', 500);
  }
}

// Create new page
export async function POST(req: NextRequest) {
  try {
    const auth = await authenticateUser(req);
    if (!auth.authorized) return auth.response; // ✅ Use authentication

    console.log(auth.user);
    // Only allow authenticated users to create pages, not visitors
    if (auth.user?.role === 'visitor') {
      return createResponse(undefined, undefined, 'Unauthorized! You are just a visitor silly', 403);
    }

    await connectToDatabase();
    const body = await req.json();

    // Validate input
    const validatedData = pageSchema.parse(body);

    // Create page
    const newPage = await Page.create({
      ...validatedData,
      updatedAt: new Date().toLocaleString("en-US"),
    });

    return createResponse(
      newPage,
      'Page created successfully',
      undefined,
      201
    );
  } catch (error) {
    if (error instanceof z.ZodError) {
      logError(error as Error, "POST /api/pages/");
      return createResponse(undefined, undefined, error.errors[0].message, 400);
    }
    logError(error as Error, "POST /api/pages/");
    return createResponse(undefined, undefined, 'Failed to create page', 500);
  }
}

// Update page
export async function PUT(req: NextRequest) {
  try {
    const auth = await authenticateUser(req);
    if (!auth.authorized) return auth.response; // ✅ Use authentication

    // Visitors are not allowed to update pages
    if (auth.user?.role === 'visitor') {
      return createResponse(undefined, undefined, 'Unauthorized!', 403);
    }

    await connectToDatabase();
    const body = await req.json();

    // Validate input
    const validatedData = pageUpdateSchema.parse(body);
    const { id, ...updates } = validatedData;

    const page = await Page.findById(id).lean<IPage>();

    if (!page) {
      return createResponse(undefined, undefined, 'Page not found', 404);
    }

    // Only the creator of the page can edit the page
    if (page && page.createdBy !== auth.user?.name && auth.user?.role !== 'admin') {
      return createResponse(undefined, undefined, 'Unauthorized!', 403);
    }

    // Update page
    const updatedPage = await Page.findByIdAndUpdate(
      id,
      {
        ...updates,
        updatedAt: new Date().toLocaleString(),
      },
      { new: true }
    ).lean();

    if (!updatedPage) {
      return createResponse(undefined, undefined, 'Page not found', 404);
    }

    return createResponse(updatedPage, 'Page updated successfully');
  } catch (error) {
    if (error instanceof z.ZodError) {
      return createResponse(undefined, undefined, error.errors[0].message, 400);
    }
    logError(error as Error, "PUT /api/pages/");
    return createResponse(undefined, undefined, 'Failed to update page', 500);
  }
}

// Delete page
export async function DELETE(req: NextRequest) {
  try {
    const auth = await authenticateUser(req);
    if (!auth.authorized) return auth.response; // ✅ Use authentication

    await connectToDatabase();
    const { id } = await req.json();

    const page = await Page.findById(id).lean<IPage>();

    if (!page) {
      return createResponse(undefined, undefined, 'Page not found', 404);
    }

    // Only the creator of the page can delete the page
    if (page && page.createdBy !== auth.user?.name && auth.user?.role !== 'admin') {
      return createResponse(undefined, undefined, 'Unauthorized!', 403);
    }

    if (!id) {
      return createResponse(undefined, undefined, 'Page ID is required', 400);
    }

    const deletedPage = await Page.findByIdAndDelete(id).lean();

    if (!deletedPage) {
      return createResponse(undefined, undefined, 'Page not found', 404);
    }

    return createResponse(undefined, 'Page deleted successfully');
  } catch (error) {
    logError(error as Error, "DELETE /api/pages/");
    return createResponse(undefined, undefined, 'Failed to delete page', 500);
  }
}

// Get page by ID (for specific page operations)
export async function PATCH(req: NextRequest) {
  try {
    const auth = await authenticateUser(req);
    if (!auth.authorized) return auth.response; // ✅ Use authentication

    if (auth.user?.role === 'visitor') {
      return createResponse(undefined, undefined, 'Unauthorized!', 403);
    }

    await connectToDatabase();
    const { id } = await req.json();

    if (!id) {
      return createResponse(undefined, undefined, 'Page ID is required', 400);
    }

    const page = await Page.findById(id).lean() as unknown;

    if (!page) {
      return createResponse(undefined, undefined, 'Page not found', 404);
    }

    return createResponse(page, 'Page retrieved successfully');
  } catch (error) {
    logError(error as Error, "DELETE /api/pages/");
    return createResponse(undefined, undefined, 'Failed to fetch page', 500);
  }
}