import { NextRequest, NextResponse } from "next/server";
import connectToDatabase from "@/utils/mongoose";
import User from "@/models/User";
import bcrypt from "bcryptjs";
import { z } from "zod";

import type { ApiResponse } from "@/types";
import { authenticateUser } from "@/utils/auth";
import { logError } from "@/utils/logger";

// Input validation schemas
const userCreateSchema = z.object({
  name: z.string().min(1, "Name is required"),
  email: z.string().email("Invalid email format"),
  password: z.string().min(8).regex(/\d/, "Must contain a number")
    .regex(/[A-Z]/, "Must contain an uppercase letter"),
  role: z.string().optional()
});

const userUpdateSchema = z.object({
  id: z.string(),
  name: z.string().optional(),
  email: z.string().email("Invalid email format").optional(),
  password: z.string().min(8)
    .regex(/\d/, "Must contain a number")
    .regex(/[A-Z]/, "Must contain an uppercase letter")
    .optional(),
  role: z.string().optional()
});

const ALLOWED_ROLES = ["visitor"] as const;
const SALT_ROUNDS = 12;

// Helper function to create standardized responses
function createResponse<T>(
  data?: T,
  message?: string,
  error?: string,
  status: number = 200
): NextResponse {
  const response: ApiResponse<T> = {
    success: !error,
    ...(data && { data }),
    ...(message && { message }),
    ...(error && { error })
  };
  return NextResponse.json(response, { status });
}

export async function GET(req: NextRequest) {
  try {
    const auth = await authenticateUser(req);
    if (!auth.authorized) return auth.response; // ✅ Use authentication

    await connectToDatabase();
    const users = await User.find({}, { password: 0 }).lean();
    return createResponse(users, "Users retrieved successfully");
  } catch (error) {
    logError(error as Error, "GET /api/users/");
    return createResponse(undefined, undefined, "Failed to fetch users", 500);
  }
}

export async function POST(req: Request) {
  try {
    await connectToDatabase();
    const body = await req.json();

    // Validate input
    const validatedData = userCreateSchema.parse(body);

    // Check for existing user
    const existingUser = await User.findOne({ email: validatedData.email });
    if (existingUser) {
      return createResponse(undefined, undefined, "User already exists", 409);
    }

    // Determine role
    const isFirstUser = (await User.countDocuments()) === 0;
    const assignedRole = isFirstUser
      ? "admin"
      : (ALLOWED_ROLES.includes(validatedData.role as typeof ALLOWED_ROLES[number])
        ? validatedData.role
        : "visitor");

    // Create user
    const hashedPassword = await bcrypt.hash(validatedData.password, SALT_ROUNDS);
    const newUser = await User.create({
      ...validatedData,
      password: hashedPassword,
      provider: "credentials",
      role: assignedRole,
    });

    // Remove password from response
    const userObj = newUser.toObject();
    delete userObj.password;
    const userWithoutPassword = userObj;

    return createResponse(
      userWithoutPassword,
      `User successfully created with role: ${assignedRole}`,
      undefined,
      201
    );
  } catch (error) {
    if (error instanceof z.ZodError) {
      logError(error as Error, "POST /api/users/");
      return createResponse(undefined, undefined, error.errors[0].message, 400);
    }
    logError(error as Error, "POST /api/users/");
    return createResponse(undefined, undefined, "Internal server error", 500);
  }
}

export async function PUT(req: NextRequest) {
  try {
    const auth = await authenticateUser(req);
    if (!auth.authorized) return auth.response; // ✅ Use authentication

    await connectToDatabase();
    const body = await req.json();

    // Validate input
    const validatedData = userUpdateSchema.parse(body);
    const { id, role, password, ...otherUpdates } = validatedData;

    // Prevent users from modifying their own role
    if (auth.user?.id === id && role) {
      return createResponse(undefined, undefined, "You cannot modify your own role", 403);
    }

    // Prevent non-admins from modifying other users
    if (auth.user?.role !== "admin" && id !== auth.user?.id) {
      return createResponse(undefined, undefined, "Unauthorized", 403);
    }

    if (auth.user?.id !== id && password) {
      if (password && password.length < 8) {
        return createResponse(undefined, undefined, "Password must be at least 8 characters", 400);
      }

      return createResponse(undefined, undefined, "You cannot modify another user's password, but nice try", 403);
    }

    if (!role && !password && Object.keys(otherUpdates).length === 0) {
      return createResponse(undefined, undefined, "No updates provided", 400);
    }

    // If password is provided, hash it
    const updates: Partial<Record<string, unknown>> = {
      role,
      ...otherUpdates,
      ...(password && {
        password: await bcrypt.hash(password, SALT_ROUNDS)
      })
    };

    const allowedUpdates = ["name", "role", "password"];
    const filteredUpdates: Partial<Record<string, unknown>> = {};
    for (const key in updates) {
      if (allowedUpdates.includes(key)) {
        filteredUpdates[key] = updates[key];
      }
    }
    console.log(filteredUpdates);

    // Update user
    const updatedUser = await User.findByIdAndUpdate(id, filteredUpdates, {
      new: true,
      select: "-password"
    }).lean();

    if (!updatedUser) {
      return createResponse(undefined, undefined, "User not found", 404);
    }

    return createResponse(updatedUser, "User updated successfully");
  } catch (error) {
    if (error instanceof z.ZodError) {
      logError(error as Error, "PUT /api/users/");
      return createResponse(undefined, undefined, error.errors[0].message, 400);
    }
    logError(error as Error, "PUT /api/users/");
    return createResponse(undefined, undefined, "Failed to update user", 500);
  }
}

export async function DELETE(req: NextRequest) {
  try {
    const auth = await authenticateUser(req);
    if (!auth.authorized) return auth.response; // ✅ Use authentication

    await connectToDatabase();
    const { id } = await req.json();

    if (!id) {
      return createResponse(undefined, undefined, "User ID is required", 400);
    }

    // Prevent users from modifying erase themselves
    if (auth.user?.id === id) {
      return createResponse(undefined, undefined, "Come on, there's so much to life yet", 403);
    }

    // Prevent non-admins from modifying other users
    if (auth.user?.role !== "admin") {
      return createResponse(undefined, undefined, "Unauthorized!", 403);
    }

    const deletedUser = await User.findByIdAndDelete(id).lean();
    if (!deletedUser) {
      return createResponse(undefined, undefined, "User not found", 404);
    }

    return createResponse(undefined, "User deleted successfully");
  } catch (error) {
    logError(error as Error, "DELETE /api/users/");
    return createResponse(undefined, undefined, "Failed to delete user", 500);
  }
}