"use client";

import { useState, useEffect, useCallback } from "react";
import Image from 'next/image';
import { CalendarClock, Trash2, UploadCloud } from 'lucide-react';
import { useDropzone } from 'react-dropzone';

interface FileInfo {
  name: string;
  url: string;
  size: number;
  type: string;
  createdAt: string;
}

const FileList = () => {
  const [files, setFiles] = useState<FileInfo[]>([]);
  const [isLoading, setIsLoading] = useState(true);
  const [error, setError] = useState<string | null>(null);

  const loadFiles = async () => {
    try {
      const res = await fetch("/api/media");
      const data = await res.json();
      if (data.success) {
        setFiles(data.data);
      } else {
        setError(data.error || "Error retrieving files");
      }
    } catch {
      setError("Failed to load files");
    } finally {
      setIsLoading(false);
    }
  };

  useEffect(() => {
    loadFiles();
  }, []);

  const handleUpload = async (file: File) => {
    const formData = new FormData();
    formData.append("file", file);

    try {
      const res = await fetch("/api/media", {
        method: "POST",
        body: formData,
      });
      const data = await res.json();

      if (data.success) {
        setFiles((prevFiles) => [...prevFiles, data.data]);
      } else {
        setError(data.error || "Upload failed");
      }
    } catch {
      setError("Failed to upload file");
    }
  };

  const onDrop = useCallback((acceptedFiles: File[]) => {
    acceptedFiles.forEach(handleUpload);
  }, []);

  const { getRootProps, getInputProps, isDragActive } = useDropzone({ onDrop });

  if (isLoading) return <div className="p-4">Loading...</div>;

  return (
    <div className="px-10 py-6">
      <div className="flex flex-col space-y-4 mb-10">
        <div className="flex justify-between items-center">
          <h1 className="text-2xl font-semibold text-gray-800">Media</h1>
        </div>
        <div {...getRootProps()} className={`border-2 border-dashed p-6 text-center cursor-pointer hover:bg-white rounded-lg ${isDragActive ? "bg-blue-200" : "bg-gray-100"}`}>
          <input {...getInputProps()} />
          <UploadCloud size={40} className={`mx-auto ${isDragActive ? "text-white font-bold" : "text-gray-500"}`} />
          <p className={`${isDragActive ? "text-white font-bold" : "text-background"}`}>
            {isDragActive ? "Drop the files here..." : "Drag and drop files here or click to select"}
          </p>
        </div>
      </div>
      {error && <div className="mb-4 p-3 bg-red-100 text-red-700 rounded">{error}</div>}
      {files.length === 0 ? (
        <p className="text-gray-500">No files uploaded yet.</p>
      ) : (
        <div className="flex flex-wrap gap-4 justify-center">
          {files.map((file) => (
            <div className="max-w-sm w-full" key={file.name}>
              <div className="flex flex-col bg-white rounded-lg shadow-lg">
                <div className="relative">
                  <button className="absolute top-2 right-2 text-white bg-alert hover:bg-red-700 flex items-center space-x-2 rounded-lg px-2 py-1 h-10 text-sm font-medium shadow-md">
                    <Trash2 size={20} />
                  </button>
                  {file.type.startsWith("image/") ? (
                    <Image alt="File preview" className="rounded-t-lg object-cover" height={200} src={file.url} width={600} />
                  ) : (
                    <div className="w-full h-[200px] flex items-center justify-center bg-gray-100 rounded-t-lg">
                      📄
                    </div>
                  )}
                </div>
                <div className="px-4 py-2 space-y-4">
                  <h3 className="text-lg font-bold">{file.name}</h3>
                  <div className="flex items-center text-background space-x-1">
                    <CalendarClock size={18} />
                    <span className="font-bold">Uploaded @:</span>
                    <span>{new Date(file.createdAt).toLocaleString()}</span>
                  </div>
                </div>
              </div>
            </div>
          ))}
        </div>
      )}
    </div>
  );
};

export default FileList;
