'use client';
import { useState, useEffect } from "react";
import { useSession } from "next-auth/react";
import Link from "next/link";
import { Trash2, Pencil, CloudUpload, CloudDownload, CalendarArrowUp, CalendarClock, CircleUserRound, AlertCircle, Loader, FilePlus } from "lucide-react";

import { PagesResponse, ApiResponse, IPage } from "@/types";
import { usePopup } from "@/components/modula/PopupProvider";

export default function PageList() {
  const { showPopup } = usePopup();
  const { data: session } = useSession();

  const [pagesData, setPages] = useState<{
    pages: IPage[],
    pagination: { total: number, page: number, limit: number, pages: number }
  }>({
    pages: [],
    pagination: { total: 0, page: 1, limit: 10, pages: 1 }
  });

  const [loading, setLoading] = useState(true);
  const [error, setError] = useState<string | null>(null);

  useEffect(() => {
    const fetchUsers = async () => {
      try {
        const response = await fetch("/api/pages");
        const dataResponse: ApiResponse<PagesResponse> = await response.json();

        if (!dataResponse.success) {
          throw new Error(dataResponse.error || "Failed to fetch pages");
        }

        setPages({
          pages: dataResponse.data?.pages || [],
          pagination: {
            total: dataResponse.data?.pagination?.total || 0,
            page: dataResponse.data?.pagination?.page || 1,
            limit: dataResponse.data?.pagination?.limit || 10,
            pages: dataResponse.data?.pagination?.pages || 1
          }
        });

      } catch (err) {
        setError(err instanceof Error ? err.message : "An error occurred");
      } finally {
        setLoading(false);
      }
    };

    fetchUsers();
  }, []);

  const deletePage = async (pageId: string) => {
    const confirmed = await showPopup("Are you sure to delete this page?", "confirm");
    if (!confirmed) {
      return;
    }

    try {
      const response = await fetch(`/api/pages/`, {
        method: "DELETE",
        headers: {
          "Content-Type": "application/json",
        },
        body: JSON.stringify({ id: pageId }),
      });

      const data: ApiResponse<null> = await response.json();

      if (data.success) {
        showPopup("Page deleted successfully", "success");

        setPages((prev) => {
          const updatedPages = prev.pages.filter((page) => page._id !== pageId);
          const updatedTotal = prev.pagination.total - 1;
          const updatedPagesCount = Math.ceil(updatedTotal / prev.pagination.limit);

          return {
            ...prev,
            pages: updatedPages,
            pagination: {
              ...prev.pagination,
              total: updatedTotal,
              pages: updatedPagesCount,
            },
          };
        });
      } else {
        showPopup(`Error deleting page: \n${data.message || data.error}`, "error");
      }
    } catch {
      showPopup("Error connecting to the server", "error");
    }
  };

  const publishPage = async (pageId: string) => {
    const confirmed = await showPopup("Are you sure to publish this page?", "confirm");
    if (!confirmed) {
      return;
    }

    try {
      const response = await fetch(`/api/pages/`, {
        method: "PUT",
        headers: {
          "Content-Type": "application/json",
        },
        body: JSON.stringify({ id: pageId, status: "published" as const }),
      });

      const data: ApiResponse<null> = await response.json();

      if (data.success) {
        showPopup("Page published successfully", "success");

        setPages((prev) => {
          const updatedPages = prev.pages.map((page) =>
            page._id === pageId ? { ...page, status: "published" as const } : page
          );

          return {
            ...prev,
            pages: updatedPages,
            pagination: {
              ...prev.pagination
            },
          };
        });
      } else {
        showPopup(`Error publishing page: \n${data.message || data.error}`, "error");
      }
    } catch (error) {
      showPopup("Error publishing to the server", "error");
      console.error("Publishing error:", error);
    }
  };

  const unpublishPage = async (pageId: string) => {
    const confirmed = await showPopup("Are you sure to unpublish this page?", "confirm");
    if (!confirmed) {
      return;
    }

    try {
      const response = await fetch(`/api/pages/`, {
        method: "PUT",
        headers: {
          "Content-Type": "application/json",
        },
        body: JSON.stringify({ id: pageId, status: "draft" as const }),
      });

      const data: ApiResponse<null> = await response.json();

      if (data.success) {
        showPopup("Page moved to drafts successfully", "success");

        setPages((prev) => {
          const updatedPages = prev.pages.map((page) =>
            page._id === pageId ? { ...page, status: "draft" as const } : page
          );

          return {
            ...prev,
            pages: updatedPages,
            pagination: {
              ...prev.pagination
            },
          };
        });
      } else {
        showPopup(`Error unpublishing page: \n${data.message || data.error}`, "error");
      }
    } catch {
      showPopup("Error unpublishing to the server", "error");
    }
  };

  if (loading) {
    return (
      <div className="flex justify-center items-center min-h-[200px]">
        <Loader className="h-8 w-8 animate-spin text-gray-500" />
      </div>
    );
  }

  if (error) {
    return (
      <div className="bg-red-100 border border-red-400 text-red-700 px-4 py-3 rounded relative">
        <div className="flex items-center">
          <AlertCircle className="h-5 w-5 mr-2" />
          <span className="font-bold">Error:</span>
        </div>
        <p className="mt-1">{error}</p>
      </div>
    );
  }

  return (
    <div className="px-10 py-6">
      <div className="flex flex-col space-y-4 mb-10">
        <div className="flex justify-between items-center">
          <h1 className="text-2xl font-semibold text-gray-800">Pages</h1>
          <div className="flex items-center space-x-4">
            <Link
              href={'/editor'}
              className="bg-primary text-white px-4 py-1 rounded-lg flex items-center"
            >
              <FilePlus size={18} /> Add Pages
            </Link>
          </div>
        </div>

        <div className="flex items-center w-full justify-end">
          <span className="text-gray-800 font-semibold mr-2">Filter by:</span>
          <select className="border border-gray-300 rounded px-2 py-1">
            <option>Page Name</option>
          </select>
        </div>
      </div>

      {
        pagesData.pages.length > 0 ? (
          <div className="">
            {pagesData.pages.map((page) => (
              <div className="max-w-sm w-full" key={page._id}>

                <div className="flex justify-end items-center mb-4 space-x-2">
                  {page.status === "published" && (session?.user?.role === 'admin' || page.createdBy === session?.user?.name) && (
                    <button className="border-2 border-yellow-500 bg-white text-yellow-500 hover:bg-gray-100 rounded-lg flex items-center space-x-2 px-2 py-1 h-10 text-sm font-medium" onClick={() => unpublishPage(page._id)}>
                      <CloudDownload size={20} /> <span className="font-bold">Unpublish</span>
                    </button>
                  )}
                  {page.status === "draft" && (session?.user?.role === 'admin' || page.createdBy === session?.user?.name) && (
                    <button className="border-2 border-green-600 bg-white text-green-600 px-2 py-1 h-10 rounded-lg flex items-center space-x-2 shadow-md text-sm font-medium" onClick={() => publishPage(page._id)}>
                      <CloudUpload size={20} /> <span className="font-bold">Publish</span>
                    </button>
                  )}
                  {(session?.user?.role === 'admin' || page.createdBy === session?.user?.name) && (
                    <Link
                      href={`/editor/${encodeURIComponent(page.title)}`}
                      className="items-center space-x-2 shadow-md text-white bg-blue-500 hover:bg-blue-700 flex justify-center rounded-lg px-2 py-1 h-10 text-sm font-medium"
                    >
                      <Pencil size={20} />
                    </Link>
                  )}
                  {(session?.user?.role === 'admin' || page.createdBy === session?.user?.name) && (
                    <button className="text-white bg-red-500 hover:bg-red-700 flex items-center space-x-2 rounded-lg px-2 py-1 h-10 text-sm font-medium shadow-md" onClick={() => deletePage(page._id)}>
                      <Trash2 size={20} />
                    </button>
                  )}
                </div>


                <div className="flex flex-col bg-white rounded-lg shadow-lg">
                  <div className="relative">
                    <img
                      alt="Website preview"
                      className="rounded-t-lg h-[200px] object-cover"
                      height="200"
                      src="https://storage.googleapis.com/a1aa/image/ZNynllU98C6hydlXjY5CoIMLnNUslVY1twrDGhbgKmI.jpg"
                      width="600"
                    />
                    <span className={`absolute top-2 right-2 bg-green-500 text-white px-2 py-1 rounded-full text-sm ${page.status === "published" ? "bg-green-500 text-white font-bold" : "bg-gray-500 text-white font-bold"}`}>
                      {page.status}
                    </span>
                  </div>


                  <div className="px-4 py-2 space-y-4">
                    <div className="w-full flex justify-between items-center">
                      <h2 className="text-2xl font-bold">
                        {page.title}
                      </h2>
                      <p className="text-gray-500">
                        /{page.title}
                      </p>
                    </div>
                    <div className="flex flex-col space-y-2">
                      <div className="flex items-center text-background space-x-1">
                        <CalendarArrowUp size={18} />
                        <span className="font-bold">
                          Created @:
                        </span>
                        <span>
                          {page.createdAt}
                        </span>
                      </div>
                      <div className="flex items-center text-background space-x-1">
                        <CalendarClock size={18} />
                        <span className="font-bold">
                          Updated @:
                        </span>
                        <span>
                          {page.updatedAt}
                        </span>
                      </div>
                      <div className="flex items-center text-background space-x-1">
                        <CircleUserRound size={18} />
                        <span className="font-bold">
                          Author:
                        </span>
                        <span>
                          {page.createdBy}
                        </span>
                      </div>
                    </div>
                  </div>
                </div>
              </div>
            ))}

            <p className="text-sm text-gray-600 mt-2">
              Page {pagesData?.pagination?.page} of {pagesData?.pagination?.pages}
            </p>
          </div>
        ) : (
          <div className="bg-yellow-100 border border-yellow-400 text-yellow-700 px-4 py-3 rounded relative">
            <span className="font-bold">No Pages Found</span>
            <p className="mt-1">There are currently no pages in the system.</p>
          </div>
        )
      }
    </div >

  );
}
