'use client';
import React, { useState, useEffect } from "react";
import { useSession } from "next-auth/react";
import { AlertCircle, Loader, Calendar, Mail, UserRoundPlus, AtSign, Trash2 } from "lucide-react";

import { ApiResponse, User } from "@/types";
import { usePopup } from "@/components/modula/PopupProvider";

export default function UsersList() {
  const { data: session } = useSession();
  const { showPopup } = usePopup();

  const [users, setUsers] = useState<User[]>([]);
  const [selectedUser, setSelectedUser] = useState<User | null>(null);
  const [newRole, setNewRole] = useState<string | null>(null);
  const [newName, setNewName] = useState<string | null>(null);

  const [loading, setLoading] = useState(true);
  const [error, setError] = useState<string | null>(null);

  const isAdmin = session?.user?.role === "admin";

  const handleUpdate = async () => {
    if (!isAdmin) {
      showPopup("You are not authorized to update users", "error");
      return
    };

    const confirmed = await showPopup("Are you sure to update this user?", "confirm");
    if (!confirmed) {
      setSelectedUser(null);
      return;
    }

    if (!selectedUser || (!newRole && !newName)) return;

    const updates: Partial<User> = {};
    if (newRole) updates.role = newRole as User["role"];
    if (newName) updates.name = newName as User["name"];

    const response = await fetch('/api/users/', {
      method: "PUT",
      headers: {
        "Content-Type": "application/json",
      },
      body: JSON.stringify({ id: selectedUser._id, ...updates }),
    });

    const data = await response.json();
    if (data.success) {
      setUsers((prevUsers) =>
        prevUsers.map((user) => (user._id === selectedUser._id ? { ...user, ...updates } : user))
      );
      setSelectedUser(null);
      showPopup("User updated successfully", "success");
    } else {
      setSelectedUser(null);
      showPopup("Failed to update user", "error");
    }
  };

  const eraseProfile = async (userId: string) => {
    if (!isAdmin) return;

    const confirmed = await showPopup("Are you sure to delete this user?", "confirm");
    if (!confirmed) {
      return;
    }

    const updates: Partial<User> = {};
    if (newRole) updates.role = newRole as User["role"];
    if (newName) updates.name = newName as User["name"];

    const response = await fetch('/api/users/', {
      method: "DELETE",
      headers: {
        "Content-Type": "application/json",
      },
      body: JSON.stringify({ id: userId }),
    });

    const data = await response.json();
    if (data.success) {
      setUsers((prevUsers) =>
        prevUsers.filter((user) => (user._id !== userId))
      );
      showPopup("User deleted successfully", "success");
    } else {
      showPopup("Failed to delete user", "error");
    }
  };

  useEffect(() => {
    const fetchUsers = async () => {
      try {
        const response = await fetch("/api/users");
        const data: ApiResponse<User[]> = await response.json();

        if (!data.success) {
          throw new Error(data.error || "Failed to fetch users");
        }

        setUsers(data.data || []);
      } catch (err) {
        setError(err instanceof Error ? err.message : "An error occurred");
      } finally {
        setLoading(false);
      }
    };

    fetchUsers();
  }, []);

  if (loading) {
    return (
      <div className="flex justify-center items-center min-h-[200px]">
        <Loader className="h-8 w-8 animate-spin text-gray-500" />
      </div>
    );
  }

  if (error) {
    return (
      <div className="bg-red-100 border border-red-400 text-red-700 px-4 py-3 rounded relative">
        <div className="flex items-center">
          <AlertCircle className="h-5 w-5 mr-2" />
          <span className="font-bold">Error:</span>
        </div>
        <p className="mt-1">{error}</p>
      </div>
    );
  }

  return (
    <div className="px-10 py-6">
      <div className="flex flex-col space-y-4 mb-10">
        <div className="flex justify-between items-center">
          <h1 className="text-2xl font-semibold text-gray-800">Users</h1>
          <div className="flex items-center space-x-4">
            <button className="bg-primary text-white px-4 py-1 rounded-lg flex items-center">
              <UserRoundPlus size={18}/> Invite Members
            </button>
          </div>
        </div>

        <div className="flex items-center w-full justify-end">
          <span className="text-gray-800 font-semibold mr-2">Filter by:</span>
          <select className="border border-gray-300 rounded px-2 py-1">
            <option>User Name</option>
          </select>
        </div>
      </div>

      <div className="flex flex-wrap gap-4 justify-center">
        {users.map((user) => (
          <div key={user._id} className="flex flex-col max-w-[400px] w-full py-2 bg-white rounded-lg shadow-md items-center justify-center">
            <div className="w-full px-5 py-2 flex space-x-2 justify-end">
              {isAdmin && session?.user?.email !== user.email && (
                <button
                  className="bg-blue-100 text-blue-700 px-3 py-1 rounded-lg text-sm font-semibold"
                  onClick={() => setSelectedUser(user)}
                >
                  Edit Role
                </button>
              )}
              {isAdmin && session?.user?.email !== user.email && (
                <button
                  className="bg-red-100 text-red-700 px-3 py-1 rounded-lg text-sm font-semibold"
                  onClick={() => eraseProfile(user._id)}
                >
                  <Trash2 size={18} />
                </button>
              )}
            </div>

            <div className="flex space-x-5 w-full px-5 py-2">
              <div className="flex-shrink-0">
                <div className="bg-blue-600 text-white rounded-lg w-16 h-16 flex items-center justify-center text-2xl font-bold">JE</div>
              </div>
              <div className="flex flex-col">
                <div className="flex justify-between items-center">
                  <h3 className="text-xl font-bold text-background">{user.name}</h3>
                </div>
                <div className="flex items-stretch space-x-2 mt-2">
                  <div className="flex flex-col justify-between">
                    <span className="text-sm font-semibold text-background">{user.role}</span>
                    <div className={`w-full h-1 rounded-lg ${user.role === "admin" ? "bg-alert" : "bg-primary"}`}></div>
                  </div>

                  <div className="w-1 bg-gray-200 rounded-lg"></div>

                  <span className="text-sm text-background flex items-center space-x-2">
                    <Calendar size={18} />
                    <strong>created: </strong> 23/1/2025
                  </span>
                </div>
              </div>
            </div>
            <div className="w-full px-5 py-2">
              <div className="flex items-center space-x-2 mt-2 text-background">
                <Mail size={18} />
                <span>jacoboeadev@gmail.com</span>
                <AtSign size={18} />
                <span className="font-bold">github</span>
              </div>
            </div>
          </div>
        ))}
      </div>

      {users.length === 0 && (
        <div className="bg-yellow-100 border border-yellow-400 text-yellow-700 px-4 py-3 rounded relative">
          <span className="font-bold">No Users Found</span>
          <p className="mt-1">There are currently no users in the system.</p>
        </div>
      )}

      {/* Edit User Modal */}
      {selectedUser && (
        <div className="fixed inset-0 flex items-center justify-center bg-black bg-opacity-50">
          <div className="bg-white p-6 rounded-lg shadow-lg w-96">
            <h2 className="text-xl font-bold text-gray-800 mb-4">Edit User</h2>

            <label className="block text-sm font-medium">Change Name:</label>
            <input
              type="text"
              className="border p-2 rounded w-full mb-4"
              defaultValue={selectedUser.name}
              onChange={(e) => setNewName(e.target.value)}
            />

            <label className="block text-sm font-medium">Change Role:</label>
            <select
              className="border p-2 rounded w-full mb-4"
              defaultValue={selectedUser.role}
              onChange={(e) => setNewRole(e.target.value)}
            >
              <option value="visitor">Visitor</option>
              <option value="colaborator">Colaborator</option>
              <option value="autor">Autor</option>
              <option value="editor">Editor</option>
              <option value="admin">Admin</option>
            </select>

            <div className="flex justify-end space-x-2">
              <button
                className="bg-green-400 hover:bg-green-500 text-white py-2 px-4 rounded-md"
                onClick={() => setSelectedUser(null)}
              >
                Cancel
              </button>
              <button
                className="bg-red-500 hover:bg-red-600 text-white py-2 px-4 rounded-md"
                onClick={handleUpdate}
              >
                Save Changes
              </button>
            </div>
          </div>
        </div>
      )}
    </div>
  );
}
