'use client';

import React, { useState, useEffect, useCallback } from "react";
import { useDrop } from "react-dnd";
import { useSession } from "next-auth/react";

import NestedDropZone from "./NestedDropZone";
import { Component, ContainerComponent, DropItem } from "@/types/componentTypes";
import { Trash2, FileSlidersIcon } from "lucide-react";

import { componentMap } from "./componentMap";
import { usePopup } from "@/components/modula/PopupProvider";
import { ApiResponse } from "@/types";
import ComponentEditor from "./ComponentEditor";

const widthClasses = {
  1: "w-full",
  2: "w-1/2",
  3: "w-1/3",
  4: "w-1/4",
  5: "w-1/5",
  6: "w-1/6",
};

/**
 * Main Canva component for rendering a drop zone canvas and managing the component hierarchy.
 */
const Canva: React.FC<{ pageEdit?: string }> = ({ pageEdit }) => {
  const [isClient, setIsClient] = useState(false);
  const [editingPage, setEditingPage] = useState(false);

  const [components, setComponents] = useState<Component[]>([]);
  const [editingComponent, setEditingComponent] = useState<Component | null>(null);

  const [showEditor, setShowEditor] = useState(false);
  const { showPopup } = usePopup();

  const [idPage, setIdPage] = useState<string | null>(null);
  const [pageTitle, setPageTitle] = useState("Untitled Page");
  const [createdDate, setCreatedDate] = useState<string | null>(null);
  const [createdBy, setCreatedBy] = useState<string | null>(null);
  const { data: session, status } = useSession();

  const fetchPageData = useCallback(async (title: string) => {
    try {
      const response = await fetch(`/api/pages/${title}`);
      const data = await response.json();

      if (data.success) {
        setIdPage(data.data._id);
        setPageTitle(data.data.title);
        setCreatedDate(data.data.createdAt);
        setCreatedBy(data.data.createdBy);
        setComponents(data.data.components || []);
      } else {
        showPopup(`Error fetching page: \n${data.message || data.error}`, "error");
      }
    } catch (error) {
      showPopup(`Error connecting to server: \n${error}`, "error");
    }
  }, [showPopup]);

  useEffect(() => {
    if (status === "authenticated") {
      if (pageEdit !== '' && pageEdit !== undefined) {
        fetchPageData(pageEdit);
        setEditingPage(true);
      }

      setCreatedDate(new Date().toLocaleString("en-US"));
      setCreatedBy(session?.user?.name || "Unknown");
    }

    setIsClient(true);
  }, [status, session, fetchPageData, pageEdit]);

  // Configure main drop zone for the canvas
  const [{ isOver }, drop] = useDrop({
    accept: "component",
    drop: (item: DropItem, monitor) => {
      if (monitor.didDrop()) return;
      addComponentToCanvas(item);
    },
    collect: (monitor) => ({
      isOver: monitor.isOver({ shallow: true }),
    }),
  });

  /**
   * Adds a new component to the canvas.
   */
  const addComponentToCanvas = (
    item: DropItem
  ) => {
    setComponents((prev) => [
      ...prev,
      createComponent(item.type as keyof typeof componentMap, item.props as Record<string, string | number | boolean | null | undefined>)
    ]);
  };

  /**
   * Creates a new component object with the given type and props.
   */
  const createComponent = (
    type: keyof typeof componentMap,
    props: Record<string, string | number | boolean | null | undefined> = {}
  ): Component => ({
    id: Date.now().toString(),
    type,
    props,
    children: [],
  });

  /**
   * Handles dropping a component inside another component.
   */
  const handleDropInside = (
    parentId: number,
    item: DropItem
  ): void => {
    setComponents((prev) => updateComponents(prev, parentId.toString(), item));
  };

  /**
   * Function to check if a component is a container component.
   */
  function isContainerComponent(component: Component): component is ContainerComponent {
    return 'children' in component;
  }

  /**
   * Updates components recursively to add a new child component.
   */
  const updateComponents = (
    components: Component[],
    parentId: string,
    item: DropItem
  ): Component[] => {
    return components
      .map((comp) => {
        if (comp.id === parentId && isContainerComponent(comp)) {
          return {
            ...comp,
            children: [
              ...comp.children,
              createComponent(item.type as keyof typeof componentMap, item.props as Record<string, string | number | boolean | null | undefined>),
            ],            
          };
        }
        if (isContainerComponent(comp)) {
          return {
            ...comp,
            children: updateComponents(comp.children, parentId, item),
          };
        }
        return comp;
      })
      .filter((comp): comp is Component => comp !== undefined);
  };

  /**
   * Updates components recursively to add a new props.
   */
  const updateProps = (id: string, key: string, newValue: string | number | boolean | null | undefined) => {
    setComponents((prevComponents) => {
      // Recursive function to update de childrens
      const updateComponentTree = (components: Component[]): Component[] => {
        return components.map((comp) => {
          if (comp.id === id.toString()) {
            const updatedComponent = { ...comp, props: { ...comp.props, [key]: newValue } };

            // If this is the component being edited, update the state
            if (editingComponent && editingComponent.id === id.toString()) {
              setEditingComponent(updatedComponent);
            }

            return updatedComponent;
          }

          // Recursively update children
          if (isContainerComponent(comp)) {
            return { ...comp, children: updateComponentTree(comp.children) };
          }
          return comp;
        });
      };

      return updateComponentTree(prevComponents);
    });
  };

  /**
   * Deletes a component by its ID.
   */
  const deleteComponent = (
    id: string
  ): void => {
    setComponents((prev) => removeComponent(prev, id));
  };

  /**
   * Removes a component by its ID recursively.
   */
  const removeComponent = (
    components: Component[],
    id: string
  ): Component[] => {
    return components
      .filter((comp) => comp.id !== id)
      .map((comp) => ({
        ...comp,
        children: isContainerComponent(comp) ? removeComponent(comp.children, id) : [],
      }));
  }


  const savePage = async () => {
    const pageData = {
      title: pageTitle,
      createdAt: createdDate,
      components: components,
      createdBy: createdBy,
    };

    if (editingPage) {
      try {
        const response = await fetch('/api/pages', {
          method: 'PUT',
          headers: {
            'Content-Type': 'application/json',
          },
          body: JSON.stringify({ id: idPage, ...pageData }),
        });

        const data: ApiResponse<{ pageId: string }> = await response.json();

        if (data.success) {
          showPopup('Page was saved! 🎉', 'success');
          return;
        } else {
          showPopup(`Error saving page: \n${data.message || data.error}`, 'error');
          return;
        }
      } catch (error) {
        showPopup(`Error connecting to server: \n${error}`, 'error');
        return;
      }
    }

    try {
      const response = await fetch('/api/pages', {
        method: 'POST',
        headers: {
          'Content-Type': 'application/json',
        },
        body: JSON.stringify(pageData),
      });

      const data: ApiResponse<{ pageId: string }> = await response.json();

      if (data.success) {
        showPopup('Page was saved! 🎉', 'success');
      } else {
        showPopup(`Error saving page: \n${data.message || data.error}`, 'error');
      }
    } catch (error) {
      showPopup(`Error connecting to server: \n${error}`, 'error');
    }
  };


  /**
   * Renders the component hierarchy recursively.
   */
  const renderComponents = (components: Component[], parent?: Component): React.ReactNode => {
    return components.map((comp) => {
      const ComponentToRender = componentMap[comp.type];
      if (!ComponentToRender) {
        return null;
      }

      const canHaveChildren = ["section", "row", "column", "parallaxSeparator",].includes(comp.type);
      const parentIsRow = parent && parent.type === "row";
      const siblingCount = parentIsRow ? parent.children.length : 1;
      const dynamicWidth = parentIsRow ? widthClasses[siblingCount as keyof typeof widthClasses] || "w-full" : "w-full";

      return (
        <div
          key={comp.id}
          className={`w-full md:${dynamicWidth} p-2 border border-dashed border-gray-400 rounded relative`}
        >
          <div className="flex flex-col justify-between items-center">
            <div className="w-full flex flex-row justify-between mb-1">

              <p className="text-gray-700">{comp.type}</p>
              <div className="w-auto flex">
                <button
                  onClick={() => { setEditingComponent(comp); setShowEditor(true); }}
                  className="ml-2 text-blue-500 text-xs flex items-center"
                >
                  <FileSlidersIcon size={20} />
                </button>
                <button
                  onClick={() => deleteComponent(comp.id)}
                  className="ml-2 text-red-500 text-xs flex items-center"
                >
                  <Trash2 size={20} />
                </button>
              </div>
            </div>

            {canHaveChildren && (
              <NestedDropZone parentId={parseInt(comp.id)} onDrop={handleDropInside}>
                <ComponentToRender embedId={""} imageHero={""} title={""} width={""} text={""} src={""} alt={""} url={""} image={""} {...comp.props}>
                  {isContainerComponent(comp) && renderComponents(comp.children, comp)}
                </ComponentToRender>
              </NestedDropZone>
            )}

            {!canHaveChildren &&
              <ComponentToRender
                embedId=""
                imageHero=""
                title=""
                width=""
                text=""
                src=""
                alt=""
                url=""
                image=""
                {...comp.props}
              >
                {null}
              </ComponentToRender>
            }


          </div>
        </div>
      );
    });
  };

  if (!isClient) {
    return null;
  }

  return (
    <div className="flex flex-col h-full w-full overflow-y-scroll">
      <div className="p-4 border-b bg-gray-100">
        <div className="flex justify-between">
          <div>
            <input
              type="text"
              value={pageTitle}
              onChange={(e) => setPageTitle(e.target.value.toLowerCase().replace(/\s+/g, "_"))}
              className="text-xl font-bold bg-transparent border-none outline-none w-full"
              placeholder="Enter page title..."
            />
            <p className="text-sm text-gray-500">Created: {createdDate}</p>
          </div>
          <div className="flex items-center">
            <button
              className="bg-green-500 px-4 py-2 rounded text-white hover:bg-green-400"
              onClick={savePage} // Llamamos a la función cuando se haga clic
            >
              Save
            </button>
          </div>
        </div>
      </div>
      {/* Main drop zone */}
      <div
        ref={drop as unknown as React.Ref<HTMLDivElement>}
        className={`flex-1 bg-white border ${isOver ? "bg-gray-100" : "bg-white"}`}
      >
        {components.length === 0 ? (
          <p className="text-gray-500 text-sm text-center m-20">
            Drag and drop components here
          </p>
        ) : (
          renderComponents(components)
        )}
      </div>


      {showEditor && editingComponent && (
        <ComponentEditor
          editingComponent={{
            ...editingComponent,
            props: editingComponent.props as Record<string, string | number | boolean | null | undefined>
          }}
          updateProps={updateProps}
          setShowEditor={setShowEditor}
        />
      )}

      {/* Display JSON representation */}
      <div className="bg-gray-100 border-t p-4 mt-4">
        <h3 className="font-bold text-lg mb-2">Generated JSON:</h3>
        <pre className="bg-white p-4 rounded border overflow-auto text-sm">
          {JSON.stringify({ title: pageTitle, createdAt: createdDate || "", components, createdBy }, null, 2)}
        </pre>
      </div>
    </div>
  );
};

export default Canva;