import { useState, useEffect } from 'react';

interface FileInfo {
  name: string;
  url: string;
  type: string;
  size: number;
  createdAt: string;
}

interface ImageSelectorProps {
  value: string;
  onChange: (value: string) => void;
  onClose: () => void;
}

const ImageSelector = ({ value, onChange, onClose }: ImageSelectorProps) => {
  const [images, setImages] = useState<FileInfo[]>([]);
  const [isLoading, setIsLoading] = useState(true);
  const [error, setError] = useState<string | null>(null);

  useEffect(() => {
    fetch("/api/media")
      .then((res) => res.json())
      .then((data) => {
        if (data.success) {
          // Filtrar solo imágenes
          const imageFiles = data.data.filter((file: FileInfo) => 
            file.type.startsWith('image/')
          );
          setImages(imageFiles);
        } else {
          setError(data.error || "Error loading images");
        }
      })
      .catch(() => setError("Failed to load images"))
      .finally(() => setIsLoading(false));
  }, []);

  return (
    <div className="fixed inset-0 bg-black bg-opacity-50 flex items-center justify-center z-50">
      <div className="bg-white rounded-lg p-4 max-w-2xl w-full max-h-[80vh] overflow-y-auto">
        <div className="flex justify-between items-center mb-4">
          <h3 className="font-bold text-lg">Select Image</h3>
          <button 
            onClick={onClose}
            className="text-gray-500 hover:text-gray-700"
          >
            ✕
          </button>
        </div>

        {isLoading ? (
          <div className="text-center py-4">Loading images...</div>
        ) : error ? (
          <div className="text-red-500 text-center py-4">{error}</div>
        ) : images.length === 0 ? (
          <div className="text-center py-4">No images available</div>
        ) : (
          <div className="grid grid-cols-3 gap-4">
            {images.map((image) => (
              <div
                key={image.url}
                onClick={() => {
                  onChange(image.url);
                  onClose();
                }}
                className={`
                  cursor-pointer p-2 border rounded-lg hover:border-blue-500 transition-colors
                  ${value === image.url ? 'border-blue-500 ring-2 ring-blue-200' : 'border-gray-200'}
                `}
              >
                <img
                  src={image.url}
                  alt={image.name}
                  className="w-full h-32 object-cover rounded"
                />
                <p className="mt-1 text-sm text-gray-600 truncate">
                  {image.name}
                </p>
              </div>
            ))}
          </div>
        )}
      </div>
    </div>
  );
};

interface ComponentProps {
  [key: string]: string | number | boolean | null | undefined;
}

interface EditingComponent {
  id: string;
  type: string;
  props: ComponentProps;
}

interface ComponentEditorProps {
  editingComponent: EditingComponent;
  updateProps: (id: string, key: string, value: string | number | boolean | null | undefined) => void;
  setShowEditor: (show: boolean) => void;
}

const ComponentEditor = ({ editingComponent, updateProps, setShowEditor }: ComponentEditorProps) => {
  const [showImageSelector, setShowImageSelector] = useState(false);
  const [selectedProp, setSelectedProp] = useState<string | null>(null);

  const isImageProp = (key: string) => {
    return key.toLowerCase().includes('image') || 
           key.toLowerCase().includes('src') || 
           key.toLowerCase().includes('avatar') ||
           key.toLowerCase().includes('logo');
  };

  return (
    <div className="absolute bottom-4 right-4 bg-white shadow-md p-4 border rounded-lg">
      <h3 className="font-bold text-lg mb-2">Edit {editingComponent.type}</h3>
      {Object.entries(editingComponent.props).map(([key, value]) => (
        <div key={key} className="mb-2">
          <label className="block text-sm font-medium text-gray-700">{key}</label>
          {isImageProp(key) ? (
            <div className="flex items-center gap-2 mt-1">
              <input
                type="text"
                value={String(value ?? '')}
                onChange={(e) => updateProps(editingComponent.id, key, e.target.value)}
                className="block w-full p-2 border border-gray-300 rounded-md shadow-sm focus:ring-blue-500 focus:border-blue-500 sm:text-sm"
              />
              <button
                onClick={() => {
                  setSelectedProp(key);
                  setShowImageSelector(true);
                }}
                className="px-3 py-2 bg-blue-50 text-blue-600 rounded hover:bg-blue-100 transition-colors"
              >
                Browse
              </button>
            </div>
          ) : (
            <input
              type="text"
              value={String(value ?? '')}
              onChange={(e) => updateProps(editingComponent.id, key, e.target.value)}
              className="mt-1 block w-full p-2 border border-gray-300 rounded-md shadow-sm focus:ring-blue-500 focus:border-blue-500 sm:text-sm"
            />
          )}
        </div>
      ))}
      <div className="flex justify-end gap-2 mt-4">
        <button 
          onClick={() => setShowEditor(false)} 
          className="px-4 py-2 text-red-500 hover:text-red-700"
        >
          Cancel
        </button>
        <button 
          onClick={() => setShowEditor(false)} 
          className="px-4 py-2 bg-green-500 text-white rounded hover:bg-green-600"
        >
          Save
        </button>
      </div>

      {showImageSelector && selectedProp && (
        <ImageSelector
          value={String(editingComponent.props[selectedProp])}
          onChange={(newValue) => {
            updateProps(editingComponent.id, selectedProp, newValue);
            setShowImageSelector(false);
          }}
          onClose={() => setShowImageSelector(false)}
        />
      )}
    </div>
  );
};

export default ComponentEditor;