'use client';
import React, { createContext, useState, useContext, ReactNode } from 'react';
import { X } from 'lucide-react';

type PopupType = 'info' | 'warning' | 'error' | 'success' | 'confirm';
type PopupConfig = {
  message: string;
  type?: PopupType;
  confirmText?: string;
  cancelText?: string;
};

type PopupContextType = {
  showPopup: (message: string, type?: PopupType) => Promise<boolean>;
};

const PopupContext = createContext<PopupContextType>({
  showPopup: async () => false,
});

export const PopupProvider = ({ children }: { children: ReactNode }) => {
  const [popup, setPopup] = useState<(PopupConfig & { resolve: (value: boolean) => void }) | null>(null);

  const showPopup = (message: string, type: PopupType = 'info'): Promise<boolean> => {
    return new Promise((resolve) => {
      setPopup({ 
        message, 
        type, 
        resolve,
        confirmText: type === 'confirm' ? 'Confirm' : 'Continue',
        cancelText: type === 'confirm' ? 'Cancel' : undefined
      });
    });
  };

  const handleConfirm = () => {
    popup?.resolve(true);
    setPopup(null);
  };

  const handleCancel = () => {
    popup?.resolve(false);
    setPopup(null);
  };

  return (
    <PopupContext.Provider value={{ showPopup }}>
      {children}
      {popup && (
        <div className="fixed inset-0 z-50 flex items-center justify-center bg-black/50">
          <div className="bg-white rounded-lg shadow-xl w-96 p-6 relative">
            <button 
              onClick={handleCancel} 
              className="absolute top-3 right-3 text-gray-500 hover:text-gray-700"
            >
              <X size={24} />
            </button>
            
            <div className={`
              mb-4 font-semibold
              ${popup.type === 'error' ? 'text-red-600' : 
                popup.type === 'warning' ? 'text-yellow-600' : 
                popup.type === 'success' ? 'text-green-600' : 
                popup.type === 'confirm' ? 'text-blue-600' : 
                'text-gray-700'}
            `}>
              {popup.message}
            </div>

            <div className="flex justify-end space-x-2 mt-4">
              {popup.type === 'confirm' && (
                <button 
                  onClick={handleCancel} 
                  className="px-4 py-2 bg-gray-200 text-gray-700 rounded hover:bg-gray-300"
                >
                  {popup.cancelText}
                </button>
              )}
              <button 
                onClick={handleConfirm} 
                className={`
                  px-4 py-2 rounded 
                  ${popup.type === 'error' ? 'bg-red-600 text-white hover:bg-red-700' : 
                    popup.type === 'warning' ? 'bg-yellow-600 text-white hover:bg-yellow-700' : 
                    popup.type === 'success' ? 'bg-green-600 text-white hover:bg-green-700' : 
                    popup.type === 'confirm' ? 'bg-blue-600 text-white hover:bg-blue-700' : 
                    'bg-gray-600 text-white hover:bg-gray-700'}
                `}
              >
                {popup.confirmText}
              </button>
            </div>
          </div>
        </div>
      )}
    </PopupContext.Provider>
  );
};

export const usePopup = () => {
  const context = useContext(PopupContext);
  if (!context) {
    throw new Error('usePopup must be used within a PopupProvider');
  }
  return context;
};